@@ -15,17 +15,53 @@ from formant.protos.model.v1 import (
     navigation_pb2,
     health_pb2,
     text_pb2,
+    intervention_pb2,
 )
 from typing_extensions import Literal
 
 from .exceptions import InvalidArgument, handle_agent_exceptions
 from .cancellable_stream_thread import CancellableStreamThread
 
+DEFUALT_IMAGE_CONTENT_TYPE = "image/jpg"
+DEFAULT_SEVERITY_TYPE = "info"
+ALLOWED_IMAGE_CONTENT_TYPES = ["image/jpg", "image/jpeg", "image/png"]
+ALLOWED_SEVERITY_TYPES = ["info", "warning", "critical", "error"]
+
 
 def current_timestamp():
     return int(time.time() * 1000)
 
 
+def set_severity_pb(
+    given,  # type: str
+):
+    severity_pb = event_pb2.INFO
+    if given == "warning":
+        severity_pb = event_pb2.WARNING
+    elif given == "critical":
+        severity_pb = event_pb2.CRITICAL
+    elif given == "error":
+        severity_pb = event_pb2.ERROR
+    return severity_pb
+
+
+def validate_literal_type(
+    given,  # type: Any
+    expected_types,  # type: List[type]
+    input_name,  # type: str
+):
+    method_name = sys._getframe(1).f_code.co_name
+    if given not in expected_types:
+        raise TypeError(
+            method_name
+            + input_name
+            + " is "
+            + given
+            + "' but must be one of: "
+            + str(expected_types)
+        )
+
+
 def validate_type(
     given,  # type: Any
     expected_types,  # type: List[type]
@@ -271,7 +307,9 @@ class Client:
             numeric_set.numerics.extend(
                 [
                     math_pb2.NumericSetEntry(
-                        value=v[0], label=k, unit=v[1] if v[1] else None,
+                        value=v[0],
+                        label=k,
+                        unit=v[1] if v[1] else None,
                     )
                 ]
             )
@@ -301,7 +339,7 @@ class Client:
         :param stream: The name of the Formant stream to post the datapoint on
         :param value: The datapoint value: raw bytes of an encoded image or frame
         :param url: The datapoint url: path to local file or valid remote URL for
-             remote files
+            remote files
         :param content_type: The format of the encoded image or frame.
             Defaults to "image/jpg".
         :param tags: Tags to include on the posted datapoint
@@ -406,7 +444,9 @@ class Client:
             datapoint_pb2.Datapoint(
                 stream=stream,
                 location=navigation_pb2.Location(
-                    latitude=latitude, longitude=longitude, altitude=altitude,
+                    latitude=latitude,
+                    longitude=longitude,
+                    altitude=altitude,
                     orientation=orientation,
                 ),
                 tags=tags if tags else {},
@@ -556,7 +596,8 @@ class Client:
         timestamp=None,  # type: Optional[int]
         end_timestamp=None,  # type: Optional[int]
         notify=False,  # type: bool
-        severity="info",  # type: Literal["info", "warning", "critical", "error"]
+        severity=DEFAULT_SEVERITY_TYPE,
+        # type: Literal["info", "warning", "critical", "error"]
     ):
         # type: (...) -> None
         """
@@ -587,14 +628,10 @@ class Client:
         """
 
         validate_type(message, [str])
+        validate_type(severity, [str])
+        validate_literal_type(severity, ALLOWED_SEVERITY_TYPES, "severity")
 
-        severity_pb = event_pb2.INFO
-        if severity == "warning":
-            severity_pb = event_pb2.WARNING
-        elif severity == "critical":
-            severity_pb = event_pb2.CRITICAL
-        elif severity == "error":
-            severity_pb = event_pb2.ERROR
+        severity_pb = set_severity_pb(severity)
 
         self.agent_stub.CreateEvent(
             agent_pb2.CreateEventRequest(
@@ -652,7 +689,9 @@ class Client:
         :param datapoint: A datapoint related to the command
         """
         response = commands_pb2.CommandResponse(
-            request_id=request_id, success=success, datapoint=datapoint,
+            request_id=request_id,
+            success=success,
+            datapoint=datapoint,
         )
         request = agent_pb2.SendCommandResponseRequest(response=response)
         self.agent_stub.SendCommandResponse(request)
@@ -686,7 +725,8 @@ class Client:
         )
 
     def unregister_command_request_callback(
-        self, f,  # type: Callable[[commands_pb2.CommandRequest], None]
+        self,
+        f,  # type: Callable[[commands_pb2.CommandRequest], None]
     ):
         # type: (...) -> None
         """
@@ -752,7 +792,8 @@ class Client:
         return self.agent_stub.GetTeleopInfo(request)
 
     def register_teleop_heartbeat_callback(
-        self, f,  # type: Callable[[agent_pb2.GetTeleopHeartbeatStreamResponse], None]
+        self,
+        f,  # type: Callable[[agent_pb2.GetTeleopHeartbeatStreamResponse], None]
     ):
         # type : (...) -> None
         """
@@ -821,7 +862,8 @@ class Client:
         )
 
     def unregister_custom_data_channel_message_callback(
-        self, f,  # type: Callable[[], None]
+        self,
+        f,  # type: Callable[[], None]
     ):
         # type : (...) -> None
         """
@@ -1076,3 +1118,223 @@ class Client:
                 channel_name_filter=channel_name_filter
             )
         )
+
+    @handle_agent_exceptions
+    def create_selection_intervention_request(
+        self,
+        title,  # type: str
+        instruction,  # type: str
+        options,  # type: List[str]
+        hint,  # type: int
+        url=None,  # type: str
+        content_type=DEFUALT_IMAGE_CONTENT_TYPE,  # type: Literal["image/jpg", "image/png"] # noqa: E501
+        timestamp=None,  # type: Optional[int]
+        severity=DEFAULT_SEVERITY_TYPE,
+        # type: Literal["info", "warning", "critical", "error"]
+    ):
+        # type : (...) -> intervention_pb2.InterventionRequest
+        """
+        Creates an intervention request based on type "selection".
+        Takes an image url, options and an integer with an optional
+        addition of instructions, and title.
+
+        :param title: The name of the intervention
+        :param instruction: The instructions detailing how to resolve the intervention
+        :param options: The list with options to select from
+        :param hint: The index of the suspected correct answer
+        :param url: The path to local file or valid remote URL for remote files
+        :param content_type: The format of the encoded image or frame.
+            Defaults to "image/jpg"
+        :param timestamp: Unix timestamp in milliseconds for the posted datapoint.
+            Uses the current time by default
+        :param severity: The severity level of the event
+        .. highlight:: python
+        .. code-block:: python
+
+            from formant.sdk.agent.v1 import Client
+
+            fclient = Client()
+            fclient.create_selection_intervention_request(
+                "Which fruit is best?",
+                "Select the best grape",
+                ["fruit_1", "fruit_2", "fruit_3"],
+                hint=1,
+                url=/home/my_user/data/test-image.jpeg
+                severity=critical
+            )
+        """
+
+        validate_type(title, [str])
+        validate_type(instruction, [str])
+        validate_type(url, [str])
+        validate_type(options, [List])
+        validate_type(hint, [int])
+        validate_type(severity, [str])
+        validate_type(content_type, [str])
+        validate_literal_type(content_type, ALLOWED_IMAGE_CONTENT_TYPES, "content_type")
+        validate_literal_type(severity, ALLOWED_SEVERITY_TYPES, "severity")
+        for option in options:
+            validate_type(option, [str])
+
+        severity_pb = set_severity_pb(severity)
+
+        request = intervention_pb2.InterventionRequest()
+        request.timestamp = timestamp if timestamp else current_timestamp()
+        request.severity = severity_pb
+        request.selection_request.title = title
+        request.selection_request.hint = hint
+        request.selection_request.instruction = instruction
+        request.selection_request.image.content_type = content_type
+        request.selection_request.image.url = url
+        request.selection_request.options.extend(options)
+        return self.agent_stub.CreateInterventionRequest(request)
+
+    @handle_agent_exceptions
+    def create_teleop_intervention_request(
+        self,
+        instruction,  # type: str
+        timestamp=None,  # type: Optional[int]
+        severity=DEFAULT_SEVERITY_TYPE,
+        # type: Literal["info", "warning", "critical", "error"]
+    ):
+        # type : (...) -> intervention_pb2.InterventionRequest
+
+        """
+        Creates an intervention request based on type "teleop".
+        requires an instruction.
+
+        :param instruction: The instructions detailing how to resolve the intervention
+        :param timestamp: Unix timestamp in milliseconds for the posted datapoint.
+            Uses the current time by default
+        :param severity: The severity level of the event
+        .. highlight:: python
+        .. code-block:: python
+
+            from formant.sdk.agent.v1 import Client
+
+            fclient = Client()
+            fclient.create_teleop_intervention_request(
+            "Get that gold",
+            )
+        """
+        validate_type(instruction, [str])
+        validate_type(severity, [str])
+        validate_literal_type(severity, ALLOWED_SEVERITY_TYPES, "severity")
+
+        severity_pb = set_severity_pb(severity)
+
+        request = intervention_pb2.InterventionRequest()
+        request.timestamp = timestamp if timestamp else current_timestamp()
+        request.severity = severity_pb
+        request.teleop_request.instruction = instruction
+        return self.agent_stub.CreateInterventionRequest(request)
+
+    @handle_agent_exceptions
+    def create_labeling_intervention_request(
+        self,
+        title,  # type: str
+        instruction,  # type: str
+        labels,  # type: Dict[str, str]
+        hint,  # type:List[intervention_pb2.LabeledPolygon]
+        url=None,  # type: str
+        content_type=DEFUALT_IMAGE_CONTENT_TYPE,  # type: Literal["image/jpg", "image/png"] # noqa: E501
+        timestamp=None,  # type: Optional[int]
+        severity=DEFAULT_SEVERITY_TYPE,
+        # type: Literal["info", "warning", "critical", "error"]
+    ):
+        # type : (...) -> intervention_pb2.InterventionRequest
+
+        """
+        :param title: The name of the intervention
+        :param instruction: The instructions detailing how to resolve the intervention
+        :param labels: An Array of labels
+        :param hint: An array of label polygons, X and Y coordinates with a label
+        :param url: The path to local file or valid remote URL for remote files
+        :param content_type: The format of the encoded image or frame.
+            Defaults to "image/jpg"
+        :param timestamp: Unix timestamp in milliseconds for the posted datapoint.
+            Uses the current time by default
+        :param severity: The severity level of the event
+
+
+        Creates an intervention request based on type "labeling".
+        it takes an image url, an instructions, 'labels'
+        which are shown as:
+                Label = {
+                        value = string;
+                        string display_name = string;
+                        }
+        and a hint which is an array of "LabeledPolygon",
+        that are shown as:
+                hint = {
+                    List of vertex,
+                    List of labels
+                }
+        and a vertex that appears as:
+                vertex = {
+                    x = float,
+                    y = float
+                }
+        """
+        validate_type(title, [str])
+        validate_type(instruction, [str])
+        validate_type(url, [str])
+        validate_type(content_type, [str])
+        validate_type(severity, [str])
+        validate_literal_type(severity, ALLOWED_SEVERITY_TYPES, "severity")
+        validate_literal_type(content_type, ALLOWED_IMAGE_CONTENT_TYPES, "content_type")
+        validate_type(labels, [List])
+        validate_type(labels, [dict])
+        for label in hint:
+            validate_type(label, [intervention_pb2.LabeledPolygon])
+        for k, v in labels.items():
+            validate_type(k, [str])
+            validate_type(v, [str])
+
+        severity_pb = set_severity_pb(severity)
+
+        request = intervention_pb2.InterventionRequest()
+        request.timestamp = timestamp if timestamp else current_timestamp()
+        request.severity = severity_pb
+        request.labeling_request.title = title
+        request.labeling_request.instruction = instruction
+        request.labeling_request.hint = hint
+        request.labeling_request.image.content_type = content_type
+        request.labeling_request.image.url = url
+        for key, value in labels.items():
+            label = intervention_pb2.Label()
+            label.value = key
+            label.display_name = value
+            request.labeling_request.labels.append(label)
+        return self.agent_stub.CreateInterventionRequest(request)
+
+    @handle_agent_exceptions
+    def get_intervention_response(
+        self,
+        request_id,  # type: str
+        timestamp=None,  # type: Optional[int]
+    ):
+        # type : (...) -> agent_pb2.GetInterventionResponse
+
+        """
+        Receives an request id, and returns a response.
+        :param request_id: The ID of the intervention request
+            to which this method responds
+        :param timestamp: Unix timestamp in milliseconds for the posted datapoint.
+            Uses the current time by default
+
+
+        .. highlight:: python
+        .. code-block:: python
+
+            from formant.sdk.agent.v1 import Client
+
+            fclient = Client()
+            request = fclient.create_teleop_intervention_request("Get that gold")
+            fclient.handle_intervention_response(request.id)
+        """
+        validate_type(request_id, [str])
+        response_request = agent_pb2.GetInterventionResponseRequest()
+        response_request.request_id = request_id
+        response_request.timestamp = timestamp if timestamp else current_timestamp()
+        return self.agent_stub.GetInterventionResponse(response_request)
