@@ -10,6 +10,8 @@ class Client:
     - Ingesting telemetry datapoints for device(s)
     - Query telemetry datapoints
     - Query stream(s) last known value
+    - Create intervention requests
+    - Create intervention responses
     Requires service account credentials (environment variables):
     - FORMANT_EMAIL
     - FORMANT_PASSWORD
@@ -262,6 +264,51 @@ class Client:
 
         return self._authenticate_request(call)
 
+    def create_intervention_response(self, params):
+        """
+        Example intervention response params:
+        {
+            "interventionId": "518e24fc-64ef-47bb-be5e-036a97aeafaa",
+            "interventionType": "teleop",
+            "data": {
+                "state": "success",
+                "notes": "looks good!"
+            }
+        }
+        """
+        def call(token):
+            headers = self._headers.copy()
+            headers["Authorization"] = "Bearer %s" % token
+            response = requests.post(
+                "%s/intervention-responses" % self._admin_api, headers=headers, json=params)
+            return response.json()
+        return self._authenticate_request(call)
+
+    def create_intervention_request(self, params):
+        """
+        Example intervention request params:
+        {
+            "message": "A teleop for a customer is requested",
+            "interventionType": "teleop",
+            "time": "2022-02-17T11:41:33.389-08:00",
+            "deviceId": "b306de84-33ca-4917-9218-f686730e24e0",
+            "tags": {},
+            "data": {
+                "instruction": "Look at the users item on the table"
+            }
+        }  
+        """
+
+        def call(token):
+            headers = self._headers.copy()
+            headers["Authorization"] = "Bearer %s" % token
+            response = requests.post(
+                "%s/intervention-requests" % self._admin_api, headers=headers, json=params
+            )
+            return response.json()
+
+        return self._authenticate_request(call)
+
     def _authenticate(self):
         payload = {
             "email": self._email,
